unit U_HashedWordCount;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, HashUnit;

type
  TForm1 = class(TForm)
    wordlist: TListBox;
    HashBtn: TButton;
    OpenDialog1: TOpenDialog;
    Memo1: TMemo;
    Timelabel: TLabel;
    IndexofBtn: TButton;
    FindBtn: TButton;
    procedure HashBtnClick(Sender: TObject);
    procedure IndexofBtnClick(Sender: TObject);
    procedure FindBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    hash:THashStr;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}
const
  numtests=10;

 function getnextword(var s,w:string):boolean;
{Note that this a "destructive" getword.
  The first word of the input string s is returned in w and
  the word is deleted from the input string}

const
  delims:set of char = [' ','.',',','(',')',':',';','"','{','}'];
var
  i:integer;
begin
  w:='';
  if length(s)>0 then
  begin
    i:=1;
    while (i<length(s))  and (s[i] in delims) do inc(i);
    delete(s,1,i-1);
    i:=1;
    while (i<=length(s)) and (not (s[i] in delims)) do inc(i);
    w:=copy(s,1,i-1);
    delete(s,1,i);
  end;
  result := (length(w) >0);
end;


type
  TCount=class(TObject)
    count:integer;
  end;

var
  starttime:int64;
procedure starttimer;
begin
  Queryperformancecounter(starttime);
end;

function stoptimer:extended;
var
  freq,stoptime:int64;
begin
  QueryPerformanceFrequency(freq);
  queryPerformancecounter(stoptime);
  result:=(stoptime-starttime)/freq;
end;

procedure TForm1.HashBtnClick(Sender: TObject);
{"Hash" operation timing}
var
  s,w:string;
  i,j:integer;
  c:TCount;
  time:extended;
begin
  If opendialog1.execute then
  begin
    wordlist.clear;
    memo1.lines.loadfromfile(opendialog1.FileName);
    caption:='File: '+opendialog1.filename;
    hash:=Thashstr.create(2013, 0.2);
    starttimer;
    for j:= 1 to numtests do
    begin
      hash.resethash;
      for i:= 0 to memo1.lines.count-1 do
      with memo1 do
      begin
        s:=lines[i];
        while getnextword(s,w) do
        begin
          {w:=lowercase(w); }
          if hash.exists(w,tobject(c))
          then c.count:=c.count+1
          else
          begin
            c:=TCount.create;
            c.count:=1;
            hash.addifnotdup(w,c);
          end;
        end;
      end;
    end;
    time:=stoptimer;
    for i:= 0 to length(hash.test)-1 do
    if hash.test[i]<>nil then
    with hash do
    begin
      c:=TCount(test[i].t);
      s:= test[i].s + '   (' + inttostr(c.count)+')';
      wordlist.items.add(s);
      c.free;
    end;
    timelabel.caption:= 'Time ='+format('%8.5f',[time])+' seconds for '
     +inttostr(numtests)+' passes through the file';
 end;
 end;

procedure TForm1.IndexofBtnClick(Sender: TObject);
{"indexof" timing }
 {Listbox items (Tstrings object) has no sort, so use indexof to
  count words and then move to sorted list at end}
var
  s,w:string;
  i,j, index:integer;
  c:TCount;
  time:extended;
begin
  If opendialog1.execute then
  begin
    wordlist.clear;
    memo1.lines.loadfromfile(opendialog1.FileName);
    caption:='File: '+opendialog1.filename;
    starttimer;
    for j:= 1 to numtests do
    begin
      wordlist.clear;
      for i:= 0 to memo1.lines.count-1 do
      with memo1 do
      begin
        s:=lines[i];
        while getnextword(s,w) do
        begin
          w:=lowercase(w);
          with wordlist.items do
          begin
            index:=wordlist.items.indexof(w);
            If index>=0 then
            TCount(objects[index]).count:=TCount(objects[index]).count+1
            else
            begin
              c:=TCount.create;
              c.count:=1;
              addobject(w,c);
            end;
          end;
        end;
      end;
    end;
    time:=stoptimer;
    for i:= 0 to wordlist.items.count-1 do
    with wordlist.Items do
    begin
      strings[i]:= strings[i] + '   (' + inttostr(TCount(objects[i]).count)+')';
      objects[i].free;
    end;
    timelabel.caption:= 'Time ='+format('%8.5f',[time])+' seconds for '
     +inttostr(numtests)+' passes through the file';
  end;
end;

procedure TForm1.FindBtnClick(Sender: TObject);
{"Find" operation timing}

var
  s,w:string;
  i,j, index:integer;
  c:TCount;
  time:extended;
  list:TStringlist;
begin
  If opendialog1.execute then
  begin
    list:=TStringlist.create;
    wordlist.clear;
    memo1.lines.loadfromfile(opendialog1.FileName);
    caption:='File: '+opendialog1.filename;
    list.sorted:=true;
    starttimer;
    for j:= 1 to numtests do
    begin
      list.clear;
      for i:= 0 to memo1.lines.count-1 do
      with memo1 do
      begin
        s:=lines[i];
        while getnextword(s,w) do
        begin
          w:=lowercase(w);
          index:=list.indexof(w);
          with list do
          If index>=0 then
            TCount(objects[index]).count:=TCount(objects[index]).count+1
           else
           begin
             c:=TCount.create;
             c.count:=1;
             addobject(w,c);
           end;
         end;
       end;
     end;

     time:=stoptimer;
     for i:= 0 to list.count-1 do wordlist.items.addobject(list[i],list.objects[i]);
     for i:=0 to wordlist.items.count-1 do
     with wordlist.Items do
     begin
       strings[i]:= strings[i] + '   (' + inttostr(TCount(objects[i]).count)+')';
       objects[i].free;
     end;
     list.free;
     timelabel.caption:= 'Time ='+format('%8.5f',[time])+' seconds for '
                        +inttostr(numtests)+' passes through the file';
   end;
 end;

end.


